@@ -1,11 +1,12 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0 WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #pragma once
 
 #include "extensionsystem_global.h"
 
 #include <aggregation/aggregate.h>
+#include <utils/filepath.h>
 #include <utils/qtcsettings.h>
 
 #include <QObject>
@@ -34,14 +35,14 @@ public:
     // Object pool operations
     static void addObject(QObject *obj);
     static void removeObject(QObject *obj);
-    static QVector<QObject *> allObjects();
+    static QObjectList allObjects();
     static QReadWriteLock *listLock();
 
     // This is useful for soft dependencies using pure interfaces.
     template <typename T> static T *getObject()
     {
         QReadLocker lock(listLock());
-        const QVector<QObject *> all = allObjects();
+        const QObjectList all = allObjects();
         for (QObject *obj : all) {
             if (T *result = qobject_cast<T *>(obj))
                 return result;
@@ -51,7 +52,7 @@ public:
     template <typename T, typename Predicate> static T *getObject(Predicate predicate)
     {
         QReadLocker lock(listLock());
-        const QVector<QObject *> all = allObjects();
+        const QObjectList all = allObjects();
         for (QObject *obj : all) {
             if (T *result = qobject_cast<T *>(obj))
                 if (predicate(result))
@@ -62,35 +63,53 @@ public:
 
     static QObject *getObjectByName(const QString &name);
 
+    static void startProfiling();
     // Plugin operations
-    static QVector<PluginSpec *> loadQueue();
+    static QList<PluginSpec *> loadQueue();
     static void loadPlugins();
-    static QStringList pluginPaths();
-    static void setPluginPaths(const QStringList &paths);
+    static void loadPluginsAtRuntime(const QSet<PluginSpec *> &plugins);
+    static Utils::FilePaths pluginPaths();
+    static void setPluginPaths(const Utils::FilePaths &paths);
     static QString pluginIID();
     static void setPluginIID(const QString &iid);
-    static const QVector<PluginSpec *> plugins();
-    static QHash<QString, QVector<PluginSpec *>> pluginCollections();
+    static const QList<PluginSpec *> plugins();
+    static QHash<QString, QList<PluginSpec *>> pluginCollections();
     static bool hasError();
     static const QStringList allErrors();
     static const QSet<PluginSpec *> pluginsRequiringPlugin(PluginSpec *spec);
-    static const QSet<PluginSpec *> pluginsRequiredByPlugin(PluginSpec *spec);
+    static const QSet<PluginSpec *> pluginsToEnableForPlugin(PluginSpec *spec);
     static void checkForProblematicPlugins();
+    static PluginSpec *specForPlugin(IPlugin *plugin);
+    static PluginSpec *specById(const QString &id);
+    static bool specExists(const QString &id);
+    static bool specExistsAndIsEnabled(const QString &id);
+
+    static void addPlugins(const QList<PluginSpec *> &specs);
+
+    static void reInstallPlugins();
+
+    static Utils::Result<> removePluginOnRestart(const QString &id);
+    static void installPluginOnRestart(
+        const Utils::FilePath &source, const Utils::FilePath &destination);
+
+    static void removePluginsAfterRestart();
+    static void installPluginsAfterRestart();
+
+    // UI
+    static std::optional<QSet<PluginSpec *>> askForEnablingPlugins(
+        QWidget *dialogParent, const QSet<PluginSpec *> &plugins, bool enable);
 
     // Settings
-    static void setSettings(Utils::QtcSettings *settings);
     static Utils::QtcSettings *settings();
-    static void setGlobalSettings(Utils::QtcSettings *settings);
     static Utils::QtcSettings *globalSettings();
     static void writeSettings();
 
     // command line arguments
     static QStringList arguments();
     static QStringList argumentsForRestart();
-    static bool parseOptions(const QStringList &args,
+    static Utils::Result<> parseOptions(const QStringList &args,
         const QMap<QString, bool> &appOptions,
-        QMap<QString, QString> *foundAppOptions,
-        QString *errorString);
+        QMap<QString, QString> *foundAppOptions);
     static void formatOptions(QTextStream &str, int optionIndentation, int descriptionIndentation);
     static void formatPluginOptions(QTextStream &str, int optionIndentation, int descriptionIndentation);
     static void formatPluginVersions(QTextStream &str);
@@ -99,7 +118,7 @@ public:
 
     static bool testRunRequested();
 
-#ifdef WITH_TESTS
+#ifdef EXTENSIONSYSTEM_WITH_TESTOPTION
     static bool registerScenario(const QString &scenarioId, std::function<bool()> scenarioStarter);
     static bool isScenarioRequested();
     static bool runScenario();
@@ -121,17 +140,19 @@ public:
     static void setCreatorProcessData(const ProcessData &data);
     static ProcessData creatorProcessData();
 
-    static void profilingReport(const char *what, const PluginSpec *spec = nullptr);
-
     static QString platformName();
 
     static bool isInitializationDone();
+    static bool isShuttingDown();
 
     static void remoteArguments(const QString &serializedArguments, QObject *socket);
     static void shutdown();
 
     static QString systemInformation();
 
+    void setAcceptTermsAndConditionsCallback(const std::function<bool(PluginSpec *)> &callback);
+    void setTermsAndConditionsAccepted(PluginSpec *spec);
+
 signals:
     void objectAdded(QObject *obj);
     void aboutToRemoveObject(QObject *obj);
