@@ -1,32 +1,12 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qmljsfindexportedcpptypes.h"
 
-#include <qmljs/qmljsinterpreter.h>
-#include <qmljs/qmljsdocument.h>
+#include "qmljsdocument.h"
+#include "qmljsinterpreter.h"
+#include "qmljstr.h"
+
 #include <cplusplus/Overview.h>
 #include <cplusplus/TypeOfExpression.h>
 #include <cplusplus/cppmodelmanagerbase.h>
@@ -34,7 +14,7 @@
 #include <utils/qtcassert.h>
 
 #include <QList>
-#include <QRegExp>
+#include <QRegularExpression>
 
 //using namespace QmlJS;
 
@@ -79,6 +59,8 @@ public:
         QmlRegisterType4,
         // int qmlRegisterType(const QUrl & url, const char * uri, int versionMajor, int versionMinor, const char * qmlName)
         QmlRegisterType5,
+        // template<typename T> inline auto qmlRegisterSingletonInstance(const char *uri, int versionMajor, int versionMinor, const char *typeName, T *cppObject)
+        QmlRegisterSingletonInstance,
         // template<typename T> int qmlRegisterSingletonType(const char * uri, int versionMajor, int versionMinor, const char * typeName, QObject *(* ) ( QQmlEngine *, QJSEngine * ) callback)
         QmlRegisterSingletonTypeCallback1,
         // int qmlRegisterSingletonType(const char * uri, int versionMajor, int versionMinor, const char * typeName, QJSValue(* ) ( QQmlEngine *, QJSEngine * ) callback)
@@ -153,12 +135,14 @@ protected:
             const Identifier *templateIdentifier = translationUnit()->identifier(templateId->identifier_token);
             if (!templateIdentifier)
                 return false;
-            const QString callName = QString::fromUtf8(templateIdentifier->chars());
-            if (callName == QLatin1String("qmlRegisterType"))
+            const QByteArray callName(templateIdentifier->chars());
+            if (callName == "qmlRegisterType")
                 registrationFunction = QmlRegisterType4;
-            else if (callName == QLatin1String("qmlRegisterSingletonType"))
+            else if (callName == "qmlRegisterSingletonInstance")
+                registrationFunction = QmlRegisterSingletonInstance;
+            else if (callName == "qmlRegisterSingletonType")
                 registrationFunction = QmlRegisterSingletonTypeCallback1;
-            else if (callName == QLatin1String("qmlRegisterUncreatableType"))
+            else if (callName == "qmlRegisterUncreatableType")
                 registrationFunction = QmlRegisterUncreatableType;
             else
                 return false;
@@ -189,6 +173,8 @@ protected:
             }
             if (fName == "qmlRegisterType")
                 registrationFunction = QmlRegisterType5;
+            else if (fName == "qmlRegisterSingletonInstance") // when called without explicit template parameter
+                registrationFunction = QmlRegisterSingletonInstance;
             else if (fName == "qmlRegisterSingletonType")
                 registrationFunction = QmlRegisterSingletonTypeCallback2;
             else if (fName == "qmlRegisterUncreatableMetaObject")
@@ -214,6 +200,7 @@ protected:
         case QmlRegisterType4:
             break;
         case QmlRegisterType5:
+        case QmlRegisterSingletonInstance:
         case QmlRegisterSingletonTypeCallback1:
         case QmlRegisterSingletonTypeCallback2:
         case QmlRegisterSingletonTypeUrl:
@@ -260,13 +247,13 @@ protected:
             nameLit = translationUnit()->stringLiteral(nameAst->literal_token);
         if (!nameLit) {
             int line, column;
-            translationUnit()->getTokenStartPosition(nameExp->firstToken(), &line, &column);
+            translationUnit()->getTokenPosition(nameExp->firstToken(), &line, &column);
             _messages += Document::DiagnosticMessage(
                         Document::DiagnosticMessage::Warning,
-                        _doc->fileName(),
+                        _doc->filePath(),
                         line, column,
-                        QmlJS::FindExportedCppTypes::tr(
-                            "The type will only be available in the QML editors when the type name is a string literal"));
+                        QmlJS::Tr::tr(
+                            "The type will only be available in the QML editors when the type name is a string literal."));
             return false;
         }
 
@@ -293,7 +280,7 @@ protected:
         }
         if (packageName.isEmpty() && _compound) {
             // check the comments in _compound for annotations
-            QRegExp uriAnnotation(QLatin1String("@uri\\s*([\\w\\.]*)"));
+            static const QRegularExpression uriAnnotation(QLatin1String("@uri\\s*([\\w\\.]*)"));
 
             // scan every comment between the pipes in
             // {|
@@ -311,8 +298,9 @@ protected:
                     continue;
                 }
                 const QString comment = stringOf(commentToken);
-                if (uriAnnotation.indexIn(comment) != -1) {
-                    packageName = uriAnnotation.cap(1);
+                const QRegularExpressionMatch match = uriAnnotation.match(comment);
+                if (match.hasMatch()) {
+                    packageName = match.captured(1);
                     break;
                 }
             }
@@ -320,13 +308,13 @@ protected:
         if (packageName.isEmpty()) {
             packageName = QmlJS::CppQmlTypes::defaultPackage;
             int line, column;
-            translationUnit()->getTokenStartPosition(ast->firstToken(), &line, &column);
+            translationUnit()->getTokenPosition(ast->firstToken(), &line, &column);
             _messages += Document::DiagnosticMessage(
                         Document::DiagnosticMessage::Warning,
-                        _doc->fileName(),
+                        _doc->filePath(),
                         line, column,
-                        QmlJS::FindExportedCppTypes::tr(
-                            "The module URI cannot be determined by static analysis. The type will be available\n"
+                        QmlJS::Tr::tr(
+                            "The module URI cannot be determined by static analysis. The type will not be available\n"
                             "globally in the QML editor. You can add a \"// @uri My.Module.Uri\" annotation to let\n"
                             "the QML editor know about a likely URI."));
         }
@@ -341,7 +329,8 @@ protected:
 
         // build the descriptor
         ExportedQmlType exportedType;
-        exportedType.isSingleton = registrationFunction == QmlRegisterSingletonTypeCallback1
+        exportedType.isSingleton = registrationFunction == QmlRegisterSingletonInstance
+                || registrationFunction == QmlRegisterSingletonTypeCallback1
                 || registrationFunction == QmlRegisterSingletonTypeCallback2
                 || registrationFunction == QmlRegisterSingletonTypeUrl;
         exportedType.isCreatable = !exportedType.isSingleton
@@ -357,7 +346,7 @@ protected:
 
         // we want to do lookup later, so also store the surrounding scope
         int line, column;
-        translationUnit()->getTokenStartPosition(ast->firstToken(), &line, &column);
+        translationUnit()->getTokenPosition(ast->firstToken(), &line, &column);
         exportedType.scope = _doc->scopeAt(line, column);
 
         if (typeId){
@@ -501,12 +490,12 @@ protected:
             nameLit = translationUnit()->stringLiteral(nameAst->literal_token);
         if (!nameLit) {
             int line, column;
-            translationUnit()->getTokenStartPosition(ast->expression_list->value->firstToken(), &line, &column);
+            translationUnit()->getTokenPosition(ast->expression_list->value->firstToken(), &line, &column);
             _messages += Document::DiagnosticMessage(
                         Document::DiagnosticMessage::Warning,
-                        _doc->fileName(),
+                        _doc->filePath(),
                         line, column,
-                        QmlJS::FindExportedCppTypes::tr(
+                        QmlJS::Tr::tr(
                             "must be a string literal to be available in the QML editor"));
             return false;
         }
@@ -515,9 +504,9 @@ protected:
         contextProperty.name = QString::fromUtf8(nameLit->chars(), nameLit->size());
         contextProperty.expression = stringOf(skipQVariant(ast->expression_list->next->value, translationUnit()));
         // we want to do lookup later, so also store the line and column of the target scope
-        translationUnit()->getTokenStartPosition(ast->firstToken(),
-                                                 &contextProperty.line,
-                                                 &contextProperty.column);
+        translationUnit()->getTokenPosition(ast->firstToken(),
+                                            &contextProperty.line,
+                                            &contextProperty.column);
 
         _contextProperties += contextProperty;
 
@@ -650,9 +639,9 @@ static QString toQmlType(const FullySpecifiedType &type)
 
 static Class *lookupClass(const QString &expression, Scope *scope, TypeOfExpression &typeOf)
 {
-    QList<LookupItem> results = typeOf(expression.toUtf8(), scope);
+    const QList<LookupItem> results = typeOf(expression.toUtf8(), scope);
     Class *klass = nullptr;
-    foreach (const LookupItem &item, results) {
+    for (const LookupItem &item : results) {
         if (item.declaration()) {
             klass = item.declaration()->asClass();
             if (klass)
@@ -708,7 +697,7 @@ static LanguageUtils::FakeMetaObject::Ptr buildFakeMetaObject(
             const FullySpecifiedType &type = propDecl->type();
             const bool isList = false; // ### fixme
             const bool isWritable = propDecl->flags() & QtPropertyDeclaration::WriteFunction;
-            const bool isPointer = type.type() && type.type()->isPointerType();
+            const bool isPointer = type.type() && type.type()->asPointerType();
             const int revision = 0; // ### fixme
             FakeMetaProperty property(
                         namePrinter.prettyName(propDecl->name()),
@@ -720,8 +709,8 @@ static LanguageUtils::FakeMetaObject::Ptr buildFakeMetaObject(
         if (QtEnum *qtEnum = member->asQtEnum()) {
             // find the matching enum
             Enum *e = nullptr;
-            QList<LookupItem> result = typeOf(namePrinter.prettyName(qtEnum->name()).toUtf8(), klass);
-            foreach (const LookupItem &item, result) {
+            const QList<LookupItem> result = typeOf(namePrinter.prettyName(qtEnum->name()).toUtf8(), klass);
+            for (const LookupItem &item : result) {
                 if (item.declaration()) {
                     e = item.declaration()->asEnum();
                     if (e)
@@ -770,7 +759,7 @@ static void buildExportedQmlObjects(
     if (cppExports.isEmpty())
         return;
 
-    foreach (const ExportedQmlType &exportedType, cppExports) {
+    for (const ExportedQmlType &exportedType : cppExports) {
         Class *klass = nullptr;
         if (!exportedType.typeExpression.isEmpty())
             klass = lookupClass(exportedType.typeExpression, exportedType.scope, typeOf);
@@ -798,7 +787,7 @@ static void buildContextProperties(
 {
     using namespace LanguageUtils;
 
-    foreach (const ContextProperty &property, contextPropertyDescriptions) {
+    for (const ContextProperty &property : contextPropertyDescriptions) {
         Scope *scope = doc->scopeAt(property.line, property.column);
         QList<LookupItem> results = typeOf(property.expression.toUtf8(), scope);
         QString typeName;
@@ -838,7 +827,7 @@ FindExportedCppTypes::FindExportedCppTypes(const CPlusPlus::Snapshot &snapshot)
 
 QStringList FindExportedCppTypes::operator()(const CPlusPlus::Document::Ptr &document)
 {
-    QTC_ASSERT(!document.isNull(), return QStringList());
+    QTC_ASSERT(!document.isNull(), return {});
 
     m_contextProperties.clear();
     m_exportedTypes.clear();
@@ -853,8 +842,7 @@ QStringList FindExportedCppTypes::operator()(const CPlusPlus::Document::Ptr &doc
     FindExportsVisitor finder(document);
     finder();
     static const QString kindKey = QLatin1String("QmlJSTools.ExportedQmlTypesDiagnostic");
-    CppModelManagerBase::trySetExtraDiagnostics(document->fileName(), kindKey,
-                                                finder.messages());
+    CppModelManagerBase::trySetExtraDiagnostics(document->filePath(), kindKey, finder.messages());
 
     // if nothing was found, done
     const QList<ContextProperty> contextPropertyDescriptions = finder.contextProperties();
@@ -864,8 +852,9 @@ QStringList FindExportedCppTypes::operator()(const CPlusPlus::Document::Ptr &doc
 
     // context properties need lookup inside function scope, and thus require a full check
     CPlusPlus::Document::Ptr localDoc = document;
-    if (document->checkMode() != CPlusPlus::Document::FullCheck && !contextPropertyDescriptions.isEmpty()) {
-        localDoc = m_snapshot.documentFromSource(document->utf8Source(), document->fileName());
+    if (document->checkMode() != CPlusPlus::Document::FullCheck
+            && !contextPropertyDescriptions.isEmpty()) {
+        localDoc = m_snapshot.documentFromSource(document->utf8Source(), document->filePath());
         localDoc->check();
     }
 
@@ -892,7 +881,7 @@ QStringList FindExportedCppTypes::operator()(const CPlusPlus::Document::Ptr &doc
         m_exportedTypes += it.value();
         fileNames += QLatin1String(it.key()->fileName());
     }
-    foreach (const LanguageUtils::FakeMetaObject::Ptr &fmo, extraFakeMetaObjects) {
+    for (const LanguageUtils::FakeMetaObject::Ptr &fmo : std::as_const(extraFakeMetaObjects)) {
         fmo->updateFingerprint();
         m_exportedTypes += fmo;
     }
@@ -914,30 +903,18 @@ bool FindExportedCppTypes::maybeExportsTypes(const CPlusPlus::Document::Ptr &doc
 {
     if (!document->control())
         return false;
-    const QByteArray qmlRegisterSingletonTypeToken("qmlRegisterType");
-    const QByteArray qmlRegisterTypeToken("qmlRegisterSingletonType");
-    const QByteArray qmlRegisterUncreatableTypeToken("qmlRegisterUncreatableType");
-    const QByteArray qmlRegisterUncreatableMetaObjectToken("qmlRegisterUncreatableMetaObject");
-    const QByteArray setContextPropertyToken("setContextProperty");
-    if (document->control()->findIdentifier(
-                qmlRegisterTypeToken.constData(), qmlRegisterTypeToken.size())) {
-        return true;
-    }
-    if (document->control()->findIdentifier(
-                qmlRegisterSingletonTypeToken.constData(), qmlRegisterSingletonTypeToken.size())) {
-        return true;
-    }
-    if (document->control()->findIdentifier(
-                qmlRegisterUncreatableTypeToken.constData(), qmlRegisterUncreatableTypeToken.size())) {
-        return true;
-    }
-    if (document->control()->findIdentifier(
-                setContextPropertyToken.constData(), setContextPropertyToken.size())) {
-        return true;
-    }
-    if (document->control()->findIdentifier(
-                qmlRegisterUncreatableMetaObjectToken.constData(), qmlRegisterUncreatableMetaObjectToken.size())) {
-        return true;
+    const QByteArray tokens[] = {
+        "qmlRegisterSingletonInstance",
+        "qmlRegisterSingletonType",
+        "qmlRegisterType",
+        "qmlRegisterUncreatableType",
+        "setContextProperty"
+        "qmlRegisterUncreatableMetaObject",
+    };
+    for (const QByteArray &token : tokens) {
+        if (document->control()->findIdentifier(token.constData(), token.size())) {
+            return true;
+        }
     }
     return false;
 }
