@@ -1,39 +1,19 @@
-/****************************************************************************
-**
-** Copyright (C) 2018 Sergey Morozov
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2018 Sergey Morozov
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+
+#include "cppchecktool.h"
 
 #include "cppcheckdiagnostic.h"
-#include "cppcheckoptions.h"
 #include "cppcheckrunner.h"
+#include "cppchecksettings.h"
 #include "cppchecktextmarkmanager.h"
-#include "cppchecktool.h"
+#include "cppchecktr.h"
 
 #include <coreplugin/messagemanager.h>
 #include <coreplugin/progressmanager/futureprogress.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 
-#include <cpptools/cppmodelmanager.h>
+#include <cppeditor/cppmodelmanager.h>
 
 #include <utils/algorithm.h>
 #include <utils/macroexpander.h>
@@ -42,11 +22,11 @@
 
 #include <QThread>
 
-namespace Cppcheck {
-namespace Internal {
+using namespace Utils;
 
-CppcheckTool::CppcheckTool(CppcheckDiagnosticManager &manager,
-                           const Utils::Id &progressId) :
+namespace Cppcheck::Internal {
+
+CppcheckTool::CppcheckTool(CppcheckDiagnosticManager &manager, const Id &progressId) :
     m_manager(manager),
     m_progressRegexp("^.* checked (\\d+)% done$"),
     m_messageRegexp("^(.+),(\\d+),(\\w+),(\\w+),(.*)$"),
@@ -59,11 +39,10 @@ CppcheckTool::CppcheckTool(CppcheckDiagnosticManager &manager,
 
 CppcheckTool::~CppcheckTool() = default;
 
-void CppcheckTool::updateOptions(const CppcheckOptions &options)
+void CppcheckTool::updateOptions(const CppcheckSettings &settings)
 {
-    m_options = options;
     m_filters.clear();
-    for (const QString &pattern : m_options.ignoredPatterns.split(',')) {
+    for (const QString &pattern : settings.ignoredPatterns().split(',')) {
         const QString trimmedPattern = pattern.trimmed();
         if (trimmedPattern.isEmpty())
             continue;
@@ -73,16 +52,16 @@ void CppcheckTool::updateOptions(const CppcheckOptions &options)
             m_filters.push_back(re);
     }
 
-    updateArguments();
+    updateArguments(settings);
 }
 
 void CppcheckTool::setProject(ProjectExplorer::Project *project)
 {
     m_project = project;
-    updateArguments();
+    updateArguments(settings());
 }
 
-void CppcheckTool::updateArguments()
+void CppcheckTool::updateArguments(const CppcheckSettings &settings)
 {
     if (!m_project)
         return;
@@ -90,53 +69,52 @@ void CppcheckTool::updateArguments()
     m_cachedAdditionalArguments.clear();
 
     QStringList arguments;
-    if (!m_options.customArguments.isEmpty()) {
+    if (!settings.customArguments().isEmpty()) {
         Utils::MacroExpander *expander = Utils::globalMacroExpander();
-        const QString expanded = expander->expand(m_options.customArguments);
+        const QString expanded = expander->expand(settings.customArguments());
         arguments.push_back(expanded);
     }
 
-    if (m_options.warning)
+    if (settings.warning())
         arguments.push_back("--enable=warning");
-    if (m_options.style)
+    if (settings.style())
         arguments.push_back("--enable=style");
-    if (m_options.performance)
+    if (settings.performance())
         arguments.push_back("--enable=performance");
-    if (m_options.portability)
+    if (settings.portability())
         arguments.push_back("--enable=portability");
-    if (m_options.information)
+    if (settings.information())
         arguments.push_back("--enable=information");
-    if (m_options.unusedFunction)
+    if (settings.unusedFunction())
         arguments.push_back("--enable=unusedFunction");
-    if (m_options.missingInclude)
+    if (settings.missingInclude())
         arguments.push_back("--enable=missingInclude");
-    if (m_options.inconclusive)
+    if (settings.inconclusive())
         arguments.push_back("--inconclusive");
-    if (m_options.forceDefines)
+    if (settings.forceDefines())
         arguments.push_back("--force");
 
-    if (!m_options.unusedFunction && !m_options.customArguments.contains("-j "))
+    if (!settings.unusedFunction() && !settings.customArguments().contains("-j "))
         arguments.push_back("-j " + QString::number(QThread::idealThreadCount()));
 
-    arguments.push_back("--template={file},{line},{severity},{id},{message}");
+    arguments.push_back("--template=\"{file},{line},{severity},{id},{message}\"");
 
-    m_runner->reconfigure(m_options.binary, arguments.join(' '));
+    m_runner->reconfigure(settings.binary.effectiveBinary(), arguments.join(' '));
 }
 
-QStringList CppcheckTool::additionalArguments(const CppTools::ProjectPart &part) const
+QStringList CppcheckTool::additionalArguments(const CppEditor::ProjectPart &part) const
 {
     QStringList result;
 
-    if (m_options.addIncludePaths) {
+    if (settings().addIncludePaths()) {
         for (const ProjectExplorer::HeaderPath &path : part.headerPaths) {
-            const QString projectDir = m_project->projectDirectory().toString();
             if (path.type == ProjectExplorer::HeaderPathType::User
-                && path.path.startsWith(projectDir))
-                result.push_back("-I " + path.path);
+                && path.path.startsWith(m_project->projectDirectory().path()))
+                result.push_back("-I " + path.path.path());
         }
     }
 
-    if (!m_options.guessArguments)
+    if (!settings().guessArguments())
         return result;
 
     using Version = Utils::LanguageVersion;
@@ -164,24 +142,20 @@ QStringList CppcheckTool::additionalArguments(const CppTools::ProjectPart &part)
         break;
     case Version::CXX98:
     case Version::CXX17:
-    case Version::CXX2a:
+    case Version::CXX20:
+    case Version::CXX23:
         result.push_back("--language=c++");
         break;
     case Version::None:
         break;
     }
 
-    if (part.qtVersion != Utils::QtVersion::None)
+    if (part.qtVersion != Utils::QtMajorVersion::None)
         result.push_back("--library=qt");
 
     return result;
 }
 
-const CppcheckOptions &CppcheckTool::options() const
-{
-    return m_options;
-}
-
 void CppcheckTool::check(const Utils::FilePaths &files)
 {
     QTC_ASSERT(m_project, return);
@@ -192,7 +166,7 @@ void CppcheckTool::check(const Utils::FilePaths &files)
     } else {
         std::copy_if(files.cbegin(), files.cend(), std::back_inserter(filtered),
                      [this](const Utils::FilePath &file) {
-            const QString stringed = file.toString();
+            const QString stringed = file.toUrlishString();
             const auto filter = [stringed](const QRegularExpression &re) {
                 return re.match(stringed).hasMatch();
             };
@@ -203,21 +177,23 @@ void CppcheckTool::check(const Utils::FilePaths &files)
     if (filtered.isEmpty())
         return;
 
-    const CppTools::ProjectInfo info = CppTools::CppModelManager::instance()->projectInfo(m_project);
-    const QVector<CppTools::ProjectPart::Ptr> parts = info.projectParts();
+    const CppEditor::ProjectInfo::ConstPtr info
+            = CppEditor::CppModelManager::projectInfo(m_project);
+    if (!info)
+        return;
+    const QList<CppEditor::ProjectPart::ConstPtr> parts = info->projectParts();
     if (parts.size() == 1) {
         QTC_ASSERT(parts.first(), return);
         addToQueue(filtered, *parts.first());
         return;
     }
 
-    std::map<CppTools::ProjectPart::Ptr, Utils::FilePaths> groups;
-    for (const Utils::FilePath &file : qAsConst(filtered)) {
-        const QString stringed = file.toString();
-        for (const CppTools::ProjectPart::Ptr &part : parts) {
-            using CppTools::ProjectFile;
+    std::map<CppEditor::ProjectPart::ConstPtr, FilePaths> groups;
+    for (const FilePath &file : std::as_const(filtered)) {
+        for (const CppEditor::ProjectPart::ConstPtr &part : parts) {
+            using CppEditor::ProjectFile;
             QTC_ASSERT(part, continue);
-            const auto match = [stringed](const ProjectFile &pFile){return pFile.path == stringed;};
+            const auto match = [file](const ProjectFile &pFile){return pFile.path == file;};
             if (Utils::contains(part->files, match))
                 groups[part].push_back(file);
         }
@@ -227,7 +203,7 @@ void CppcheckTool::check(const Utils::FilePaths &files)
         addToQueue(group.second, *group.first);
 }
 
-void CppcheckTool::addToQueue(const Utils::FilePaths &files, CppTools::ProjectPart &part)
+void CppcheckTool::addToQueue(const Utils::FilePaths &files, const CppEditor::ProjectPart &part)
 {
     const QString key = part.id();
     if (!m_cachedAdditionalArguments.contains(key))
@@ -243,14 +219,14 @@ void CppcheckTool::stop(const Utils::FilePaths &files)
 
 void CppcheckTool::startParsing()
 {
-    if (m_options.showOutput) {
-        const QString message = tr("Cppcheck started: \"%1\".").arg(m_runner->currentCommand());
+    if (settings().showOutput()) {
+        const QString message = Tr::tr("Cppcheck started: \"%1\".").arg(m_runner->currentCommand());
         Core::MessageManager::writeSilently(message);
     }
 
     m_progress = std::make_unique<QFutureInterface<void>>();
     const Core::FutureProgress *progress = Core::ProgressManager::addTask(
-                m_progress->future(), QObject::tr("Cppcheck"), m_progressId);
+                m_progress->future(), Tr::tr("Cppcheck"), m_progressId);
     QObject::connect(progress, &Core::FutureProgress::canceled,
                      this, [this]{stop({});});
     m_progress->setProgressRange(0, 100);
@@ -262,7 +238,7 @@ void CppcheckTool::parseOutputLine(const QString &line)
     if (line.isEmpty())
         return;
 
-    if (m_options.showOutput)
+    if (settings().showOutput())
         Core::MessageManager::writeSilently(line);
 
     enum Matches { Percentage = 1 };
@@ -293,7 +269,7 @@ void CppcheckTool::parseErrorLine(const QString &line)
     if (line.isEmpty())
         return;
 
-    if (m_options.showOutput)
+    if (settings().showOutput())
         Core::MessageManager::writeSilently(line);
 
     enum Matches { File = 1, Line, Severity, Id, Message };
@@ -318,12 +294,19 @@ void CppcheckTool::parseErrorLine(const QString &line)
 
 void CppcheckTool::finishParsing()
 {
-    if (m_options.showOutput)
-        Core::MessageManager::writeSilently(tr("Cppcheck finished."));
+    if (settings().showOutput())
+        Core::MessageManager::writeSilently(Tr::tr("Cppcheck finished."));
 
     QTC_ASSERT(m_progress, return);
     m_progress->reportFinished();
 }
 
-} // namespace Internal
-} // namespace Cppcheck
+void CppcheckTool::finishWithFail(const QString &exitMessage)
+{
+    if (!exitMessage.isEmpty())
+        Core::MessageManager::writeSilently(exitMessage);
+    QTC_ASSERT(m_progress, return);
+    m_progress->cancelAndFinish();
+}
+
+} // Cppcheck::Internal
