@@ -58,8 +58,6 @@
 #include <projectexplorer/devicesupport/devicekitaspects.h>
 #include <projectexplorer/devicesupport/idevice.h>
 #include <projectexplorer/devicesupport/deviceprocessesdialog.h>
-#include <projectexplorer/devicesupport/sshparameters.h>
-#include <projectexplorer/itaskhandler.h>
 #include <projectexplorer/kitchooser.h>
 #include <projectexplorer/kitmanager.h>
 #include <projectexplorer/project.h>
@@ -82,7 +80,6 @@
 #include <texteditor/texteditorsettings.h>
 
 #include <utils/algorithm.h>
-#include <utils/appmainwindow.h>
 #include <utils/basetreeview.h>
 #include <utils/checkablemessagebox.h>
 #include <utils/fancymainwindow.h>
@@ -93,7 +90,6 @@
 #include <utils/qtcassert.h>
 #include <utils/statuslabel.h>
 #include <utils/stringutils.h>
-#include <utils/styledbar.h>
 #include <utils/temporarydirectory.h>
 #include <utils/utilsicons.h>
 #include <utils/winutils.h>
@@ -123,8 +119,6 @@
 #include <QVBoxLayout>
 #include <QVariant>
 
-#include <climits>
-
 #define DEBUG_STATE 1
 #ifdef DEBUG_STATE
 //#   define STATE_DEBUG(s)
@@ -440,6 +434,10 @@ void addStandardActions(QWidget *treeView, QMenu *menu)
 
     menu->addSeparator();
 
+    addAction(view, menu, Tr::tr("Copy Content to Clipboard"), true, [view] {
+        setClipboardAndSelection(view->contentAsText());
+    });
+
     addAction(view, menu, Tr::tr("Copy Selected Items to Clipboard"), true, [view] {
         setClipboardAndSelection(view->selectionAsText());
     });
@@ -565,6 +563,19 @@ static Kit *findUniversalCdbKit()
     return KitManager::kit(cdbPredicate());
 }
 
+class DebugServerToolFactory : public DeviceToolAspectFactory
+{
+public:
+    DebugServerToolFactory()
+    {
+        setToolId(Constants::DEBUGSERVER_TOOL_ID);
+        setToolType(DeviceToolAspect::RunTool);
+        setFilePattern({"gdbserver"});
+        setLabelText(Tr::tr("GDB server executable:"));
+        setToolTip(Tr::tr("The GDB server executable to use on the device."));
+    }
+};
+
 ///////////////////////////////////////////////////////////////////////
 //
 // DebuggerPluginPrivate
@@ -651,7 +662,6 @@ public:
 
     void setOrRemoveBreakpoint();
     void enableOrDisableBreakpoint();
-    void updateDebugWithoutDeployMenu();
 
     void attachToRunningApplication();
     void attachToUnstartedApplicationDialog();
@@ -722,6 +732,8 @@ public:
 
     std::optional<QPoint> attachToUnstartedApplicationDialogLastPosition;
 
+    DebugServerToolFactory m_debugServerToolFactory;
+
     // FIXME: Needed?
 //            QString mainScript = runConfig->property("mainScript").toString();
 //            const bool isDebuggableScript = mainScript.endsWith(".py"); // Only Python for now.
@@ -1208,8 +1220,9 @@ DebuggerPluginPrivate::DebuggerPluginPrivate(const QStringList &arguments)
             }
         });
 
-    connect(ProjectExplorerPlugin::instance(), &ProjectExplorerPlugin::settingsChanged,
-        this, &DebuggerPluginPrivate::updateDebugWithoutDeployMenu);
+    globalProjectExplorerSettings().deployBeforeRun.addOnChanged(this, [this] {
+        m_debugWithoutDeployAction.setVisible(globalProjectExplorerSettings().deployBeforeRun());
+    });
 
     // Debug mode setup
     m_mode = new DebugMode;
@@ -1448,9 +1461,7 @@ bool DebuggerPluginPrivate::parseArgument(QStringList::const_iterator &it,
         }
         rp.setUseTerminal(useTerminal);
 
-        auto debugger = createDebuggerWorker(runControl, rp);
-        Q_UNUSED(debugger)
-
+        runControl->setRunRecipe(debuggerRecipe(runControl, rp));
         m_scheduledStarts.append(runControl);
         return true;
     }
@@ -1480,9 +1491,7 @@ bool DebuggerPluginPrivate::parseArgument(QStringList::const_iterator &it,
             return false;
         }
 
-        auto debugger = createDebuggerWorker(runControl, rp);
-        Q_UNUSED(debugger)
-
+        runControl->setRunRecipe(debuggerRecipe(runControl, rp));
         m_scheduledStarts.append(runControl);
         return true;
     }
@@ -1519,19 +1528,11 @@ void DebuggerPluginPrivate::updatePresetState()
     if (PluginManager::isShuttingDown())
         return;
 
-    Project *startupProject = ProjectManager::startupProject();
-    RunConfiguration *startupRunConfig = activeRunConfigForActiveProject();
     DebuggerEngine *currentEngine = EngineManager::currentEngine();
 
     const auto canRun = ProjectExplorerPlugin::canRunStartupProject(
         ProjectExplorer::Constants::DEBUG_RUN_MODE);
 
-    QString startupRunConfigName;
-    if (startupRunConfig)
-        startupRunConfigName = startupRunConfig->displayName();
-    if (startupRunConfigName.isEmpty() && startupProject)
-        startupRunConfigName = startupProject->displayName();
-
     // Restrict width, otherwise Creator gets too wide, see QTCREATORBUG-21885
     const QString startToolTip = canRun ? Tr::tr("Start debugging of startup project")
                                         : canRun.error();
@@ -1655,9 +1656,7 @@ void DebuggerPluginPrivate::attachToLastCore()
     rp.setStartMode(AttachToCore);
     rp.setCloseMode(DetachAtClose);
 
-    auto debugger = createDebuggerWorker(runControl, rp);
-    Q_UNUSED(debugger)
-
+    runControl->setRunRecipe(debuggerRecipe(runControl, rp));
     runControl->start();
 }
 
@@ -1676,7 +1675,7 @@ void DebuggerPluginPrivate::attachToRunningApplication()
     kitChooser->setShowIcons(true);
 
     auto dlg = new DeviceProcessesDialog(kitChooser, ICore::dialogParent());
-    dlg->addAcceptButton(Tr::tr("&Attach to Process"));
+    dlg->addAcceptButton(msgAttachToProcess());
     dlg->showAllDevices();
     if (dlg->exec() == QDialog::Rejected) {
         delete dlg;
@@ -1709,9 +1708,7 @@ void DebuggerPluginPrivate::attachToRunningApplication()
         rp.setUseContinueInsteadOfRun(true);
         rp.setContinueAfterAttach(false);
 
-        auto debugger = createDebuggerWorker(runControl, rp);
-        Q_UNUSED(debugger)
-
+        runControl->setRunRecipe(debuggerRecipe(runControl, rp));
         runControl->start();
     }
 }
@@ -1780,9 +1777,7 @@ RunControl *DebuggerPluginPrivate::attachToRunningProcess(Kit *kit,
     rp.setCloseMode(DetachAtClose);
     rp.setContinueAfterAttach(contAfterAttach);
 
-    auto debugger = createDebuggerWorker(runControl, rp);
-    Q_UNUSED(debugger)
-
+    runControl->setRunRecipe(debuggerRecipe(runControl, rp));
     runControl->start();
     return runControl;
 }
@@ -1952,12 +1947,6 @@ void DebuggerPluginPrivate::setInitialState()
     settings().expandStack.setEnabled(false);
 }
 
-void DebuggerPluginPrivate::updateDebugWithoutDeployMenu()
-{
-    const bool state = projectExplorerSettings().deployBeforeRun;
-    m_debugWithoutDeployAction.setVisible(state);
-}
-
 void DebuggerPluginPrivate::dumpLog()
 {
     DebuggerEngine *engine = EngineManager::currentEngine();
@@ -2206,13 +2195,13 @@ bool wantRunTool(ToolMode toolMode, const QString &toolName)
         QString currentMode;
         switch (buildType) {
             case BuildConfiguration::Debug:
-                currentMode = Tr::tr("Debug");
+                currentMode = msgBuildConfigurationDebug();
                 break;
             case BuildConfiguration::Profile:
-                currentMode = Tr::tr("Profile");
+                currentMode = msgBuildConfigurationProfile();
                 break;
             case BuildConfiguration::Release:
-                currentMode = Tr::tr("Release");
+                currentMode = msgBuildConfigurationRelease();
                 break;
             default:
                 QTC_CHECK(false);
@@ -2341,9 +2330,7 @@ void DebuggerPlugin::attachExternalApplication(RunControl *rc)
     rp.setStartMode(AttachToLocalProcess);
     rp.setCloseMode(DetachAtClose);
 
-    auto debugger = createDebuggerWorker(runControl, rp);
-    Q_UNUSED(debugger)
-
+    runControl->setRunRecipe(debuggerRecipe(runControl, rp));
     runControl->start();
 }
 
