@@ -1,33 +1,12 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "sourceagent.h"
 
 #include "debuggerengine.h"
 #include "debuggericons.h"
 #include "debuggerinternalconstants.h"
+#include "debuggertr.h"
 #include "stackhandler.h"
 
 #include <coreplugin/editormanager/editormanager.h>
@@ -50,8 +29,7 @@
 using namespace Core;
 using namespace TextEditor;
 
-namespace Debugger {
-namespace Internal {
+namespace Debugger::Internal {
 
 class SourceAgentPrivate
 {
@@ -62,23 +40,21 @@ public:
 public:
     QPointer<BaseTextEditor> editor;
     QPointer<DebuggerEngine> engine;
-    TextMark *locationMark;
+    TextMark *locationMark = nullptr;
     QString path;
     QString producer;
 };
 
 SourceAgentPrivate::SourceAgentPrivate()
-  : editor(0)
-  , locationMark(0)
-  , producer(QLatin1String("remote"))
+  : producer("remote")
 {
 }
 
 SourceAgentPrivate::~SourceAgentPrivate()
 {
     if (editor)
-        EditorManager::closeDocument(editor->document());
-    editor = 0;
+        EditorManager::closeDocuments({editor->document()});
+    editor = nullptr;
     delete locationMark;
 }
 
@@ -91,7 +67,7 @@ SourceAgent::SourceAgent(DebuggerEngine *engine)
 SourceAgent::~SourceAgent()
 {
     delete d;
-    d = 0;
+    d = nullptr;
 }
 
 void SourceAgent::setSourceProducerName(const QString &name)
@@ -105,8 +81,8 @@ void SourceAgent::setContent(const QString &filePath, const QString &content)
     d->path = filePath;
 
     if (!d->editor) {
-        QString titlePattern = d->producer + QLatin1String(": ")
-            + Utils::FileName::fromString(filePath).fileName();
+        QString titlePattern = d->producer + ": "
+            + Utils::FilePath::fromString(filePath).fileName();
         d->editor = qobject_cast<BaseTextEditor *>(
             EditorManager::openEditorWithContents(
                 CppEditor::Constants::CPPEDITOR_ID,
@@ -121,7 +97,7 @@ void SourceAgent::setContent(const QString &filePath, const QString &content)
         EditorManager::activateEditor(d->editor);
     }
 
-    QPlainTextEdit *plainTextEdit = d->editor->editorWidget();
+    Utils::PlainTextEdit *plainTextEdit = d->editor->editorWidget();
     QTC_ASSERT(plainTextEdit, return);
     plainTextEdit->setReadOnly(true);
 
@@ -135,12 +111,14 @@ void SourceAgent::updateLocationMarker()
     if (d->locationMark)
         d->editor->textDocument()->removeMark(d->locationMark);
     delete d->locationMark;
-    d->locationMark = 0;
-    if (d->engine->stackHandler()->currentFrame().file == d->path) {
+    d->locationMark = nullptr;
+    if (d->engine->stackHandler()->currentFrame().file == Utils::FilePath::fromString(d->path)) {
         int lineNumber = d->engine->stackHandler()->currentFrame().line;
 
-        d->locationMark = new TextMark(QString(), lineNumber,
-                                       Constants::TEXT_MARK_CATEGORY_LOCATION);
+        d->locationMark = new TextMark(Utils::FilePath(),
+                                       lineNumber,
+                                       {Tr::tr("Debugger Location"),
+                                        Constants::TEXT_MARK_CATEGORY_LOCATION});
         d->locationMark->setIcon(Icons::LOCATION.icon());
         d->locationMark->setPriority(TextMark::HighPriority);
 
@@ -153,5 +131,4 @@ void SourceAgent::updateLocationMarker()
     }
 }
 
-} // namespace Internal
-} // namespace Debugger
+} // Debugger::Internal
