@@ -24,9 +24,13 @@
 #include <QMutexLocker>
 #include <QVariantList>
 
-#include <limits>
 #include <memory>
 
+#ifdef WITH_TESTS
+#include <QSignalSpy>
+#include <QTest>
+#endif
+
 using namespace Utils;
 
 namespace ProjectExplorer {
@@ -56,88 +60,34 @@ public:
         return devices;
     }
 
-    static DeviceManager *clonedInstance;
-
     mutable QMutex mutex;
     QList<IDevice::Ptr> devices;
     QHash<Id, Id> defaultDevices;
     PersistentSettingsWriter *writer = nullptr;
 };
-DeviceManager *DeviceManagerPrivate::clonedInstance = nullptr;
 
 } // namespace Internal
 
 using namespace Internal;
 
-DeviceManager *DeviceManager::m_instance = nullptr;
+static DeviceManager *m_instance = nullptr;
+static std::unique_ptr<DeviceManagerPrivate> d;
 
 DeviceManager *DeviceManager::instance()
 {
     return m_instance;
 }
 
-int DeviceManager::deviceCount() const
+int DeviceManager::deviceCount()
 {
     return d->devices.count();
 }
 
-void DeviceManager::replaceInstance()
-{
-    const QList<Id> newIds =
-        Utils::transform(DeviceManagerPrivate::clonedInstance->d->devices, &IDevice::id);
-
-    for (const IDevice::Ptr &dev : std::as_const(m_instance->d->devices)) {
-        if (!newIds.contains(dev->id()))
-            dev->aboutToBeRemoved();
-    }
-
-    {
-        QMutexLocker locker(&instance()->d->mutex);
-        copy(DeviceManagerPrivate::clonedInstance, instance(), false);
-    }
-
-    emit instance()->deviceListReplaced();
-    emit instance()->updated();
-}
-
-void DeviceManager::removeClonedInstance()
-{
-    delete DeviceManagerPrivate::clonedInstance;
-    DeviceManagerPrivate::clonedInstance = nullptr;
-}
-
-DeviceManager *DeviceManager::cloneInstance()
-{
-    QTC_ASSERT(!DeviceManagerPrivate::clonedInstance, return nullptr);
-
-    DeviceManagerPrivate::clonedInstance = new DeviceManager(false);
-    copy(instance(), DeviceManagerPrivate::clonedInstance, true);
-    return DeviceManagerPrivate::clonedInstance;
-}
-
-DeviceManager *DeviceManager::clonedInstance()
-{
-    return DeviceManagerPrivate::clonedInstance;
-}
-
-void DeviceManager::copy(const DeviceManager *source, DeviceManager *target, bool deep)
-{
-    if (deep) {
-        for (const IDevice::Ptr &device : std::as_const(source->d->devices))
-            target->d->devices << device->clone();
-    } else {
-        target->d->devices = source->d->devices;
-    }
-    target->d->defaultDevices = source->d->defaultDevices;
-}
-
 void DeviceManager::save()
 {
-    if (d->clonedInstance == this || !d->writer)
-        return;
     Store data;
     data.insert(DeviceManagerKey, variantFromStore(toMap()));
-    d->writer->save(data, Core::ICore::dialogParent());
+    d->writer->save(data);
 }
 
 static FilePath settingsFilePath(const QString &extension)
@@ -163,15 +113,28 @@ void DeviceManager::load()
     QList<IDevice::Ptr> sdkDevices;
     if (reader.load(systemSettingsFilePath("devices.xml")))
         sdkDevices = fromMap(storeFromVariant(reader.restoreValues().value(DeviceManagerKey)), &defaultDevices);
+    for (IDevice::Ptr &device : sdkDevices)
+        device->setFromSdk();
     // read devices file from user settings path
     QList<IDevice::Ptr> userDevices;
     if (reader.load(settingsFilePath("devices.xml")))
         userDevices = fromMap(storeFromVariant(reader.restoreValues().value(DeviceManagerKey)), &defaultDevices);
     // Insert devices into the model. Prefer the higher device version when there are multiple
     // devices with the same id.
-    for (IDevice::ConstPtr device : std::as_const(userDevices)) {
+    for (IDevice::Ptr device : std::as_const(userDevices)) {
+        // Make sure devices removed via the sdktool really disappear.
+        if (device->isFromSdk()) {
+            const bool stillPresent
+                = Utils::contains(sdkDevices, [id = device->id()](const IDevice::Ptr &sdkDev) {
+                      return sdkDev->id() == id;
+                  });
+            if (!stillPresent)
+                continue;
+        }
+
         for (const IDevice::Ptr &sdkDevice : std::as_const(sdkDevices)) {
             if (device->id() == sdkDevice->id() || device->rootPath() == sdkDevice->rootPath()) {
+                device->setFromSdk(); // For pre-17 settings.
                 if (device->version() < sdkDevice->version())
                     device = sdkDevice;
                 sdkDevices.removeOne(sdkDevice);
@@ -191,7 +154,11 @@ void DeviceManager::load()
             d->defaultDevices[device->type()] = device->id();
     }
 
-    emit devicesLoaded();
+    // Trigger auto-connection
+    for (const IDevice::Ptr &device : d->devices)
+        device->postLoad();
+
+    emit m_instance->devicesLoaded();
 }
 
 static const IDeviceFactory *restoreFactory(const Store &map)
@@ -232,7 +199,7 @@ QList<IDevice::Ptr> DeviceManager::fromMap(const Store &map, QHash<Id, Id> *defa
     return devices;
 }
 
-Store DeviceManager::toMap() const
+Store DeviceManager::toMap()
 {
     Store map;
     Store defaultDeviceMap;
@@ -241,16 +208,19 @@ Store DeviceManager::toMap() const
 
     map.insert(DefaultDevicesKey, variantFromStore(defaultDeviceMap));
     QVariantList deviceList;
-    for (const IDevice::Ptr &device : std::as_const(d->devices))
-        deviceList << variantFromStore(device->toMap());
+    for (const IDevice::Ptr &device : std::as_const(d->devices)) {
+        Store store;
+        device->toMap(store);
+        if (store.isEmpty())
+            continue;
+        deviceList << variantFromStore(store);
+    }
     map.insert(DeviceListKey, deviceList);
     return map;
 }
 
-void DeviceManager::addDevice(const IDevice::ConstPtr &_device)
+void DeviceManager::addDevice(const IDevice::Ptr &device)
 {
-    const IDevice::Ptr device = _device->clone();
-
     QStringList names;
     for (const IDevice::Ptr &tmp : std::as_const(d->devices)) {
         if (tmp->id() != device->id())
@@ -258,41 +228,39 @@ void DeviceManager::addDevice(const IDevice::ConstPtr &_device)
     }
 
     // TODO: make it thread safe?
-    device->settings()->displayName.setValue(
-        Utils::makeUniquelyNumbered(device->displayName(), names));
+    device->setDisplayName(Utils::makeUniquelyNumbered(device->displayName(), names));
 
     const int pos = d->indexForId(device->id());
 
     if (!defaultDevice(device->type()))
         d->defaultDevices.insert(device->type(), device->id());
-    if (this == DeviceManager::instance() && d->clonedInstance)
-        d->clonedInstance->addDevice(device->clone());
 
     if (pos >= 0) {
         {
             QMutexLocker locker(&d->mutex);
             d->devices[pos] = device;
         }
-        emit deviceUpdated(device->id());
+        emit m_instance->deviceUpdated(device->id());
     } else {
         {
             QMutexLocker locker(&d->mutex);
             d->devices << device;
         }
-        emit deviceAdded(device->id());
+        emit m_instance->deviceAdded(device->id());
 
         if (FSEngine::isAvailable())
             FSEngine::addDevice(device->rootPath());
     }
 
-    emit updated();
+    emit m_instance->updated();
 }
 
 void DeviceManager::removeDevice(Id id)
 {
     const IDevice::Ptr device = mutableDevice(id);
     QTC_ASSERT(device, return);
-    QTC_ASSERT(this != instance() || device->isAutoDetected(), return);
+
+    device->aboutToBeRemoved();
 
     const bool wasDefault = d->defaultDevices.value(device->type()) == device->id();
     const Id deviceType = device->type();
@@ -300,7 +268,7 @@ void DeviceManager::removeDevice(Id id)
         QMutexLocker locker(&d->mutex);
         d->devices.removeAt(d->indexForId(id));
     }
-    emit deviceRemoved(device->id());
+    emit m_instance->deviceRemoved(device->id());
 
     if (FSEngine::isAvailable())
         FSEngine::removeDevice(device->rootPath());
@@ -309,24 +277,19 @@ void DeviceManager::removeDevice(Id id)
         for (int i = 0; i < d->devices.count(); ++i) {
             if (deviceAt(i)->type() == deviceType) {
                 d->defaultDevices.insert(deviceAt(i)->type(), deviceAt(i)->id());
-                emit deviceUpdated(deviceAt(i)->id());
+                emit m_instance->deviceUpdated(deviceAt(i)->id());
                 break;
             }
         }
     }
-    if (this == instance() && d->clonedInstance)
-        d->clonedInstance->removeDevice(id);
 
-    emit updated();
+    emit m_instance->updated();
 }
 
 void DeviceManager::setDeviceState(Id deviceId, IDevice::DeviceState deviceState)
 {
-    // To see the state change in the DeviceSettingsWidget. This has to happen before
-    // the pos check below, in case the device is only present in the cloned instance.
-    if (this == instance() && d->clonedInstance)
-        d->clonedInstance->setDeviceState(deviceId, deviceState);
-
+    if (!d || !m_instance)
+        return;
     const int pos = d->indexForId(deviceId);
     if (pos < 0)
         return;
@@ -336,11 +299,11 @@ void DeviceManager::setDeviceState(Id deviceId, IDevice::DeviceState deviceState
 
     // TODO: make it thread safe?
     device->setDeviceState(deviceState);
-    emit deviceUpdated(deviceId);
-    emit updated();
+    emit m_instance->deviceUpdated(deviceId);
+    emit m_instance->updated();
 }
 
-bool DeviceManager::isLoaded() const
+bool DeviceManager::isLoaded()
 {
     return d->writer;
 }
@@ -348,7 +311,7 @@ bool DeviceManager::isLoaded() const
 // Thread safe
 IDevice::ConstPtr DeviceManager::deviceForPath(const FilePath &path)
 {
-    const QList<IDevice::Ptr> devices = instance()->d->deviceList();
+    const QList<IDevice::Ptr> devices = d->deviceList();
 
     if (path.scheme() == u"device") {
         for (const IDevice::Ptr &dev : devices) {
@@ -373,32 +336,26 @@ IDevice::ConstPtr DeviceManager::defaultDesktopDevice()
 
 void DeviceManager::setDefaultDevice(Id id)
 {
-    QTC_ASSERT(this != instance(), return);
-
     const IDevice::ConstPtr &device = find(id);
     QTC_ASSERT(device, return);
     const IDevice::ConstPtr &oldDefaultDevice = defaultDevice(device->type());
     if (device == oldDefaultDevice)
         return;
     d->defaultDevices.insert(device->type(), device->id());
-    emit deviceUpdated(device->id());
-    emit deviceUpdated(oldDefaultDevice->id());
+    emit m_instance->deviceUpdated(device->id());
+    emit m_instance->deviceUpdated(oldDefaultDevice->id());
 
-    emit updated();
+    emit m_instance->updated();
 }
 
-DeviceManager::DeviceManager(bool isInstance) : d(std::make_unique<DeviceManagerPrivate>())
+DeviceManager::DeviceManager()
 {
-    QTC_ASSERT(isInstance == !m_instance, return);
-
-    if (!isInstance)
-        return;
-
+    d = std::make_unique<DeviceManagerPrivate>();
     m_instance = this;
     connect(Core::ICore::instance(), &Core::ICore::saveSettingsRequested,
             this, &DeviceManager::save);
 
-    DeviceFileHooks &deviceHooks = DeviceFileHooks::instance();
+    DeviceFileHooks deviceHooks;
 
     deviceHooks.isSameDevice = [](const FilePath &left, const FilePath &right) {
         auto leftDevice = DeviceManager::deviceForPath(left);
@@ -407,15 +364,15 @@ DeviceManager::DeviceManager(bool isInstance) : d(std::make_unique<DeviceManager
         return leftDevice == rightDevice;
     };
 
-    deviceHooks.localSource = [](const FilePath &file) -> expected_str<FilePath> {
+    deviceHooks.localSource = [](const FilePath &file) -> Result<FilePath> {
         auto device = DeviceManager::deviceForPath(file);
         if (!device)
             return make_unexpected(Tr::tr("No device for path \"%1\"").arg(file.toUserOutput()));
         return device->localSource(file);
     };
 
-    deviceHooks.fileAccess = [](const FilePath &filePath) -> expected_str<DeviceFileAccess *> {
-        if (!filePath.needsDevice())
+    deviceHooks.fileAccess = [](const FilePath &filePath) -> Result<DeviceFileAccess *> {
+        if (filePath.isLocal())
             return DesktopDeviceFileAccess::instance();
         IDevice::ConstPtr device = DeviceManager::deviceForPath(filePath);
         if (!device) {
@@ -430,7 +387,7 @@ DeviceManager::DeviceManager(bool isInstance) : d(std::make_unique<DeviceManager
         return fileAccess;
     };
 
-    deviceHooks.environment = [](const FilePath &filePath) -> expected_str<Environment> {
+    deviceHooks.environment = [](const FilePath &filePath) -> Result<Environment> {
         auto device = DeviceManager::deviceForPath(filePath);
         if (!device) {
             return make_unexpected(
@@ -465,6 +422,8 @@ DeviceManager::DeviceManager(bool isInstance) : d(std::make_unique<DeviceManager
         return device->osType();
     };
 
+    DeviceFileHooks::setupDeviceFileHooks(deviceHooks);
+
     DeviceProcessHooks processHooks;
 
     processHooks.processImplHook = [](const FilePath &filePath) -> ProcessInterface * {
@@ -478,19 +437,18 @@ DeviceManager::DeviceManager(bool isInstance) : d(std::make_unique<DeviceManager
 
 DeviceManager::~DeviceManager()
 {
-    if (d->clonedInstance != this)
-        delete d->writer;
-    if (m_instance == this)
-        m_instance = nullptr;
+    delete d->writer;
+    m_instance = nullptr;
+    d.reset();
 }
 
-IDevice::ConstPtr DeviceManager::deviceAt(int idx) const
+IDevice::Ptr DeviceManager::deviceAt(int idx)
 {
-    QTC_ASSERT(idx >= 0 && idx < deviceCount(), return IDevice::ConstPtr());
+    QTC_ASSERT(idx >= 0 && idx < deviceCount(), return IDevice::Ptr());
     return d->devices.at(idx);
 }
 
-void DeviceManager::forEachDevice(const std::function<void(const IDeviceConstPtr &)> &func) const
+void DeviceManager::forEachDevice(const std::function<void(const IDeviceConstPtr &)> &func)
 {
     const QList<IDevice::Ptr> devices = d->deviceList();
 
@@ -498,48 +456,40 @@ void DeviceManager::forEachDevice(const std::function<void(const IDeviceConstPtr
         func(device);
 }
 
-IDevice::Ptr DeviceManager::mutableDevice(Id id) const
+IDevice::Ptr DeviceManager::mutableDevice(Id id)
 {
     const int index = d->indexForId(id);
     return index == -1 ? IDevice::Ptr() : d->devices.at(index);
 }
 
-bool DeviceManager::hasDevice(const QString &name) const
+bool DeviceManager::hasDevice(const QString &name)
 {
     return Utils::anyOf(d->devices, [&name](const IDevice::Ptr &device) {
         return device->displayName() == name;
     });
 }
 
-IDevice::ConstPtr DeviceManager::find(Id id) const
+IDevice::Ptr DeviceManager::find(Id id)
 {
     const int index = d->indexForId(id);
-    return index == -1 ? IDevice::ConstPtr() : deviceAt(index);
+    return index == -1 ? IDevice::Ptr() : deviceAt(index);
 }
 
-IDevice::ConstPtr DeviceManager::defaultDevice(Id deviceType) const
+IDevice::Ptr DeviceManager::defaultDevice(Id deviceType)
 {
     const Id id = d->defaultDevices.value(deviceType);
-    return id.isValid() ? find(id) : IDevice::ConstPtr();
+    return id.isValid() ? find(id) : IDevice::Ptr();
 }
 
-} // namespace ProjectExplorer
-
-
 #ifdef WITH_TESTS
-#include <projectexplorer/projectexplorer_test.h>
-#include <QSignalSpy>
-#include <QTest>
-#include <QUuid>
-
-namespace ProjectExplorer {
+namespace Internal {
 
 class TestDevice : public IDevice
 {
 public:
     TestDevice()
     {
-        setupId(AutoDetected, Id::fromString(QUuid::createUuid().toString()));
+        setupId(AutoDetected, Id::generate());
         setType(testTypeId());
         setMachineType(Hardware);
         setOsType(HostOsInfo::hostOs());
@@ -561,96 +511,94 @@ public:
     }
 };
 
-void ProjectExplorerTest::testDeviceManager()
+class DeviceManagerTest : public QObject
 {
-    TestDeviceFactory factory;
+    Q_OBJECT
 
-    TestDevice::Ptr dev = IDevice::Ptr(new TestDevice);
-    dev->settings()->displayName.setValue(QLatin1String("blubbdiblubbfurz!"));
-    QVERIFY(dev->isAutoDetected());
-    QCOMPARE(dev->deviceState(), IDevice::DeviceStateUnknown);
-    QCOMPARE(dev->type(), TestDevice::testTypeId());
+private slots:
 
-    TestDevice::Ptr dev2 = dev->clone();
-    QCOMPARE(dev->id(), dev2->id());
+    void test()
+    {
+        TestDeviceFactory factory;
 
-    DeviceManager * const mgr = DeviceManager::instance();
-    QVERIFY(!mgr->find(dev->id()));
-    const int oldDeviceCount = mgr->deviceCount();
+        TestDevice::Ptr dev = IDevice::Ptr(new TestDevice);
+        dev->setDisplayName(QLatin1String("blubbdiblubbfurz!"));
+        QVERIFY(dev->isAutoDetected());
+        QCOMPARE(dev->deviceState(), IDevice::DeviceStateUnknown);
+        QCOMPARE(dev->type(), TestDevice::testTypeId());
 
-    QSignalSpy deviceAddedSpy(mgr, &DeviceManager::deviceAdded);
-    QSignalSpy deviceRemovedSpy(mgr, &DeviceManager::deviceRemoved);
-    QSignalSpy deviceUpdatedSpy(mgr, &DeviceManager::deviceUpdated);
-    QSignalSpy deviceListReplacedSpy(mgr, &DeviceManager::deviceListReplaced);
-    QSignalSpy updatedSpy(mgr, &DeviceManager::updated);
+        QVERIFY(!DeviceManager::find(dev->id()));
+        const int oldDeviceCount = DeviceManager::deviceCount();
 
-    mgr->addDevice(dev);
-    QCOMPARE(mgr->deviceCount(), oldDeviceCount + 1);
-    QVERIFY(mgr->find(dev->id()));
-    QVERIFY(mgr->hasDevice(dev->displayName()));
-    QCOMPARE(deviceAddedSpy.count(), 1);
-    QCOMPARE(deviceRemovedSpy.count(), 0);
-    QCOMPARE(deviceUpdatedSpy.count(), 0);
-    QCOMPARE(deviceListReplacedSpy.count(), 0);
-    QCOMPARE(updatedSpy.count(), 1);
-    deviceAddedSpy.clear();
-    updatedSpy.clear();
+        DeviceManager * const mgr = DeviceManager::instance();
+        QSignalSpy deviceAddedSpy(mgr, &DeviceManager::deviceAdded);
+        QSignalSpy deviceRemovedSpy(mgr, &DeviceManager::deviceRemoved);
+        QSignalSpy deviceUpdatedSpy(mgr, &DeviceManager::deviceUpdated);
+        QSignalSpy updatedSpy(mgr, &DeviceManager::updated);
 
-    mgr->setDeviceState(dev->id(), IDevice::DeviceStateUnknown);
-    QCOMPARE(deviceAddedSpy.count(), 0);
-    QCOMPARE(deviceRemovedSpy.count(), 0);
-    QCOMPARE(deviceUpdatedSpy.count(), 0);
-    QCOMPARE(deviceListReplacedSpy.count(), 0);
-    QCOMPARE(updatedSpy.count(), 0);
+        DeviceManager::addDevice(dev);
+        QCOMPARE(DeviceManager::deviceCount(), oldDeviceCount + 1);
+        QVERIFY(DeviceManager::find(dev->id()));
+        QVERIFY(DeviceManager::hasDevice(dev->displayName()));
+        QCOMPARE(deviceAddedSpy.count(), 1);
+        QCOMPARE(deviceRemovedSpy.count(), 0);
+        QCOMPARE(deviceUpdatedSpy.count(), 0);
+        QCOMPARE(updatedSpy.count(), 1);
+        deviceAddedSpy.clear();
+        updatedSpy.clear();
 
-    mgr->setDeviceState(dev->id(), IDevice::DeviceReadyToUse);
-    QCOMPARE(mgr->find(dev->id())->deviceState(), IDevice::DeviceReadyToUse);
-    QCOMPARE(deviceAddedSpy.count(), 0);
-    QCOMPARE(deviceRemovedSpy.count(), 0);
-    QCOMPARE(deviceUpdatedSpy.count(), 1);
-    QCOMPARE(deviceListReplacedSpy.count(), 0);
-    QCOMPARE(updatedSpy.count(), 1);
-    deviceUpdatedSpy.clear();
-    updatedSpy.clear();
+        DeviceManager::setDeviceState(dev->id(), IDevice::DeviceStateUnknown);
+        QCOMPARE(deviceAddedSpy.count(), 0);
+        QCOMPARE(deviceRemovedSpy.count(), 0);
+        QCOMPARE(deviceUpdatedSpy.count(), 0);
+        QCOMPARE(updatedSpy.count(), 0);
 
-    mgr->addDevice(dev2);
-    QCOMPARE(mgr->deviceCount(), oldDeviceCount + 1);
-    QVERIFY(mgr->find(dev->id()));
-    QCOMPARE(deviceAddedSpy.count(), 0);
-    QCOMPARE(deviceRemovedSpy.count(), 0);
-    QCOMPARE(deviceUpdatedSpy.count(), 1);
-    QCOMPARE(deviceListReplacedSpy.count(), 0);
-    QCOMPARE(updatedSpy.count(), 1);
-    deviceUpdatedSpy.clear();
-    updatedSpy.clear();
+        DeviceManager::setDeviceState(dev->id(), IDevice::DeviceReadyToUse);
+        QCOMPARE(DeviceManager::find(dev->id())->deviceState(), IDevice::DeviceReadyToUse);
+        QCOMPARE(deviceAddedSpy.count(), 0);
+        QCOMPARE(deviceRemovedSpy.count(), 0);
+        QCOMPARE(deviceUpdatedSpy.count(), 1);
+        QCOMPARE(updatedSpy.count(), 1);
+        deviceUpdatedSpy.clear();
+        updatedSpy.clear();
 
-    TestDevice::Ptr dev3 = IDevice::Ptr(new TestDevice);
-    QVERIFY(dev->id() != dev3->id());
+        TestDevice::Ptr dev3 = IDevice::Ptr(new TestDevice);
+        QVERIFY(dev->id() != dev3->id());
 
-    dev3->settings()->displayName.setValue(dev->displayName());
-    mgr->addDevice(dev3);
-    QCOMPARE(mgr->deviceAt(mgr->deviceCount() - 1)->displayName(),
-             QString(dev3->displayName() + QLatin1Char('2')));
-    QCOMPARE(deviceAddedSpy.count(), 1);
-    QCOMPARE(deviceRemovedSpy.count(), 0);
-    QCOMPARE(deviceUpdatedSpy.count(), 0);
-    QCOMPARE(deviceListReplacedSpy.count(), 0);
-    QCOMPARE(updatedSpy.count(), 1);
-    deviceAddedSpy.clear();
-    updatedSpy.clear();
+        dev3->setDisplayName(dev->displayName());
+        DeviceManager::addDevice(dev3);
+        QCOMPARE(
+            DeviceManager::deviceAt(DeviceManager::deviceCount() - 1)->displayName(),
+            QString(dev->displayName() + QLatin1Char('2')));
+        QCOMPARE(deviceAddedSpy.count(), 1);
+        QCOMPARE(deviceRemovedSpy.count(), 0);
+        QCOMPARE(deviceUpdatedSpy.count(), 0);
+        QCOMPARE(updatedSpy.count(), 1);
+        deviceAddedSpy.clear();
+        updatedSpy.clear();
 
-    mgr->removeDevice(dev->id());
-    mgr->removeDevice(dev3->id());
-    QCOMPARE(mgr->deviceCount(), oldDeviceCount);
-    QVERIFY(!mgr->find(dev->id()));
-    QVERIFY(!mgr->find(dev3->id()));
-    QCOMPARE(deviceAddedSpy.count(), 0);
-    QCOMPARE(deviceRemovedSpy.count(), 2);
-//    QCOMPARE(deviceUpdatedSpy.count(), 0); Uncomment once the "default" stuff is gone.
-    QCOMPARE(deviceListReplacedSpy.count(), 0);
-    QCOMPARE(updatedSpy.count(), 2);
+        DeviceManager::removeDevice(dev->id());
+        DeviceManager::removeDevice(dev3->id());
+        QCOMPARE(DeviceManager::deviceCount(), oldDeviceCount);
+        QVERIFY(!DeviceManager::find(dev->id()));
+        QVERIFY(!DeviceManager::find(dev3->id()));
+        QCOMPARE(deviceAddedSpy.count(), 0);
+        QCOMPARE(deviceRemovedSpy.count(), 2);
+        //    QCOMPARE(deviceUpdatedSpy.count(), 0); Uncomment once the "default" stuff is gone.
+        QCOMPARE(updatedSpy.count(), 2);
+    }
+};
+
+QObject *createDeviceManagerTest()
+{
+    return new DeviceManagerTest;
 }
 
+} // namespace Internal
+#endif // WITH_TESTS
+
 } // namespace ProjectExplorer
 
-#endif // WITH_TESTS
+#ifdef WITH_TESTS
+#include <devicemanager.moc>
+#endif
