@@ -1,51 +1,31 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "defaultpropertyprovider.h"
 
-#include "qbskitinformation.h"
+#include "qbskitaspect.h"
 #include "qbsprojectmanagerconstants.h"
-
-#include <coreplugin/messagemanager.h>
-#include <baremetal/baremetalconstants.h>
-#include <projectexplorer/abi.h>
-#include <projectexplorer/gcctoolchain.h>
-#include <projectexplorer/kit.h>
-#include <projectexplorer/kitinformation.h>
-#include <projectexplorer/toolchain.h>
-#include <projectexplorer/projectexplorerconstants.h>
-#include <projectexplorer/msvctoolchain.h>
-
-#include <utils/hostosinfo.h>
-#include <utils/qtcassert.h>
+#include "qbsprojectmanagertr.h"
 
 #include <android/androidconstants.h>
+#include <coreplugin/messagemanager.h>
+#include <baremetal/baremetalconstants.h>
 #include <ios/iosconstants.h>
+#include <projectexplorer/abi.h>
+#include <projectexplorer/devicesupport/devicekitaspects.h>
+#include <projectexplorer/devicesupport/idevice.h>
+#include <projectexplorer/gcctoolchain.h>
+#include <projectexplorer/kit.h>
+#include <projectexplorer/toolchain.h>
+#include <projectexplorer/sysrootkitaspect.h>
+#include <projectexplorer/toolchainkitaspect.h>
+#include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/msvctoolchain.h>
 #include <qtsupport/baseqtversion.h>
-#include <qtsupport/qtkitinformation.h>
-#include <winrt/winrtconstants.h>
+#include <qtsupport/qtkitaspect.h>
+#include <utils/hostosinfo.h>
+#include <utils/qtcassert.h>
+#include <webassembly/webassemblyconstants.h>
 
 #include <QDir>
 #include <QFileInfo>
@@ -60,7 +40,6 @@ using namespace Constants;
 namespace Internal {
 using namespace ProjectExplorer::Constants;
 using namespace Ios::Constants;
-using namespace WinRt::Internal::Constants;
 
 static QString extractToolchainPrefix(QString *compilerName)
 {
@@ -81,16 +60,19 @@ static QString extractToolchainPrefix(QString *compilerName)
 
 static QString targetPlatform(const ProjectExplorer::Abi &abi, const ProjectExplorer::Kit *k)
 {
-    const Utils::Id device = ProjectExplorer::DeviceTypeKitAspect::deviceTypeId(k);
+    const Utils::Id deviceType = ProjectExplorer::RunDeviceTypeKitAspect::deviceTypeId(k);
+    if (deviceType == WebAssembly::Constants::WEBASSEMBLY_DEVICE_TYPE)
+        return "wasm-emscripten";
+
     switch (abi.os()) {
     case ProjectExplorer::Abi::WindowsOS:
         return QLatin1String("windows");
    case ProjectExplorer::Abi::DarwinOS:
-        if (device == DESKTOP_DEVICE_TYPE)
+        if (deviceType == DESKTOP_DEVICE_TYPE)
             return QLatin1String("macos");
-        if (device == IOS_DEVICE_TYPE)
+        if (deviceType == IOS_DEVICE_TYPE)
             return QLatin1String("ios");
-        if (device == IOS_SIMULATOR_TYPE)
+        if (deviceType == IOS_SIMULATOR_TYPE)
             return QLatin1String("ios-simulator");
         return QLatin1String("darwin");
     case ProjectExplorer::Abi::LinuxOS:
@@ -124,34 +106,40 @@ static QString targetPlatform(const ProjectExplorer::Abi &abi, const ProjectExpl
     return QString();
 }
 
-static QStringList toolchainList(const ProjectExplorer::ToolChain *tc)
+static QString toolchainType(const ProjectExplorer::Toolchain *tc)
 {
-    QStringList list;
-    if (tc->typeId() == ProjectExplorer::Constants::CLANG_TOOLCHAIN_TYPEID
-            || (tc->typeId() == Android::Constants::ANDROID_TOOLCHAIN_TYPEID
-                && tc->compilerCommand().toString().contains("clang"))) {
-        list << QLatin1String("clang") << QLatin1String("llvm") << QLatin1String("gcc");
-    } else if (tc->typeId() == ProjectExplorer::Constants::GCC_TOOLCHAIN_TYPEID
-               || tc->typeId() == Android::Constants::ANDROID_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("gcc"); // TODO: Detect llvm-gcc
-    } else if (tc->typeId() == ProjectExplorer::Constants::MINGW_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("mingw") << QLatin1String("gcc");
-    } else if (tc->typeId() == ProjectExplorer::Constants::CLANG_CL_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("clang-cl") << QLatin1String("msvc");
-    } else if (tc->typeId() == ProjectExplorer::Constants::MSVC_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("msvc");
-    } else if (tc->typeId() == BareMetal::Constants::IAREW_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("iar");
-    } else if (tc->typeId() == BareMetal::Constants::KEIL_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("keil");
-    } else if (tc->typeId() == BareMetal::Constants::SDCC_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("sdcc");
+    const Utils::Id type = tc->typeId();
+    if (type == ProjectExplorer::Constants::CLANG_TOOLCHAIN_TYPEID
+            || (type == Android::Constants::ANDROID_TOOLCHAIN_TYPEID
+                && tc->compilerCommand().fileName().contains("clang"))) {
+        return "clang";
     }
-    return list;
+    if (type == ProjectExplorer::Constants::GCC_TOOLCHAIN_TYPEID
+               || type == Android::Constants::ANDROID_TOOLCHAIN_TYPEID) {
+        return "gcc"; // TODO: Detect llvm-gcc
+    }
+    if (type == ProjectExplorer::Constants::MINGW_TOOLCHAIN_TYPEID)
+        return "mingw";
+    if (type == ProjectExplorer::Constants::CLANG_CL_TOOLCHAIN_TYPEID)
+        return "clang-cl";
+    if (type == ProjectExplorer::Constants::MSVC_TOOLCHAIN_TYPEID)
+        return "msvc";
+    if (type == BareMetal::Constants::IAREW_TOOLCHAIN_TYPEID)
+        return "iar";
+    if (type == BareMetal::Constants::KEIL_TOOLCHAIN_TYPEID)
+        return "keil";
+    if (type == BareMetal::Constants::SDCC_TOOLCHAIN_TYPEID)
+        return "sdcc";
+    if (type == WebAssembly::Constants::WEBASSEMBLY_TOOLCHAIN_TYPEID)
+        return "emscripten";
+    return {};
 }
 
 static QString architecture(const ProjectExplorer::Abi &targetAbi)
 {
+    if (targetAbi.architecture() == Abi::AsmJsArchitecture)
+        return "wasm";
+
     if (targetAbi.architecture() != ProjectExplorer::Abi::UnknownArchitecture) {
         QString architecture = ProjectExplorer::Abi::toString(targetAbi.architecture());
 
@@ -167,6 +155,10 @@ static QString architecture(const ProjectExplorer::Abi &targetAbi)
                 else
                     architecture += "v7a";
                 return architecture;
+            case ProjectExplorer::Abi::LoongArchArchitecture:
+                if (targetAbi.wordWidth() == 64)
+                    architecture += "_64";
+                return architecture;
             default:
                 break;
             }
@@ -177,10 +169,11 @@ static QString architecture(const ProjectExplorer::Abi &targetAbi)
             switch (targetAbi.architecture()) {
             case ProjectExplorer::Abi::X86Architecture:
                 architecture.append(QLatin1Char('_'));
-                // fall through
+                [[fallthrough]];
             case ProjectExplorer::Abi::ArmArchitecture:
                 // ARM sub-architectures are currently not handled, which is kind of problematic
             case ProjectExplorer::Abi::MipsArchitecture:
+            case ProjectExplorer::Abi::LoongArchArchitecture:
             case ProjectExplorer::Abi::PowerPCArchitecture:
                 architecture.append(QString::number(targetAbi.wordWidth()));
                 break;
@@ -195,14 +188,14 @@ static QString architecture(const ProjectExplorer::Abi &targetAbi)
     return QString();
 }
 
-static bool isMultiTargetingToolchain(const ProjectExplorer::ToolChain *tc)
+static bool isMultiTargetingToolchain(const ProjectExplorer::Toolchain *tc)
 {
     // Clang and QCC are multi-targeting compilers; others (GCC/MinGW, MSVC, ICC) are not
     return tc->targetAbi().os() == ProjectExplorer::Abi::QnxOS
             || tc->typeId() == ProjectExplorer::Constants::CLANG_TOOLCHAIN_TYPEID;
 }
 
-static QStringList architectures(const ProjectExplorer::ToolChain *tc)
+static QStringList architectures(const ProjectExplorer::Toolchain *tc)
 {
     // For platforms which can have builds for multiple architectures in a single configuration
     // (Darwin, Android), regardless of whether the toolchain is multi-targeting or not (Clang
@@ -240,8 +233,7 @@ static void filterCompilerLinkerFlags(const ProjectExplorer::Abi &targetAbi, QSt
 {
     for (int i = 0; i < flags.size(); ) {
         if (targetAbi.architecture() != ProjectExplorer::Abi::UnknownArchitecture
-                && flags[i] == QStringLiteral("-arch")
-                && i + 1 < flags.size()) {
+            && (flags[i] == "-arch" || flags[i] == "-target") && i + 1 < flags.size()) {
             flags.removeAt(i);
             flags.removeAt(i);
         } else {
@@ -259,12 +251,12 @@ QVariantMap DefaultPropertyProvider::autoGeneratedProperties(const ProjectExplor
     if (!sysroot.isEmpty())
         data.insert(QLatin1String(QBS_SYSROOT), sysroot);
 
-    ToolChain *tcC = ToolChainKitAspect::cToolChain(k);
-    ToolChain *tcCxx = ToolChainKitAspect::cxxToolChain(k);
+    Toolchain *tcC = ToolchainKitAspect::cToolchain(k);
+    Toolchain *tcCxx = ToolchainKitAspect::cxxToolchain(k);
     if (!tcC && !tcCxx)
         return data;
 
-    ToolChain *mainTc = tcCxx ? tcCxx : tcC;
+    Toolchain *mainTc = tcCxx ? tcCxx : tcC;
 
     Abi targetAbi = mainTc->targetAbi();
 
@@ -279,9 +271,9 @@ QVariantMap DefaultPropertyProvider::autoGeneratedProperties(const ProjectExplor
     }
     data.insert(QLatin1String(QBS_TARGETPLATFORM), targetPlatform(targetAbi, k));
 
-    QStringList toolchain = toolchainList(mainTc);
+    QString toolchain = toolchainType(mainTc);
     if (targetAbi.osFlavor() == Abi::AndroidLinuxFlavor) {
-        const IDevice::ConstPtr dev = DeviceKitAspect::device(k);
+        const IDevice::ConstPtr dev = RunDeviceKitAspect::device(k);
         if (dev) {
             const QString sdkDir = k->value(Android::Constants::ANDROID_KIT_SDK).toString();
             if (!sdkDir.isEmpty())
@@ -334,12 +326,12 @@ QVariantMap DefaultPropertyProvider::autoGeneratedProperties(const ProjectExplor
         if (!mainToolchainPrefix.isEmpty())
             data.insert(QLatin1String(CPP_TOOLCHAINPREFIX), mainToolchainPrefix);
 
-        if (toolchain.contains(QLatin1String("clang-cl"))) {
+        if (toolchain == "clang-cl") {
             data.insert(QLatin1String(CPP_COMPILERNAME), mainCompilerName);
             const auto clangClToolchain =
-                    static_cast<ProjectExplorer::Internal::ClangClToolChain *>(mainTc);
+                    static_cast<ProjectExplorer::Internal::ClangClToolchain *>(mainTc);
             data.insert(QLatin1String(CPP_VCVARSALLPATH), clangClToolchain->varsBat());
-        } else if (toolchain.contains(QLatin1String("msvc"))) {
+        } else if (toolchain == "msvc") {
             data.insert(QLatin1String(CPP_COMPILERNAME), mainCompilerName);
         } else {
             if (!mainCompilerName.isEmpty())
@@ -353,11 +345,11 @@ QVariantMap DefaultPropertyProvider::autoGeneratedProperties(const ProjectExplor
         if (tcC && tcCxx && !cCompilerPath.isEmpty() && !cxxCompilerPath.isEmpty()
                 && cCompilerPath.absolutePath() != cxxCompilerPath.absolutePath()) {
             Core::MessageManager::writeFlashing(
-                tr("C and C++ compiler paths differ. C compiler may not work."));
+                Tr::tr("C and C++ compiler paths differ. C compiler may not work."));
         }
-        data.insert(QLatin1String(CPP_TOOLCHAINPATH), mainFilePath.absolutePath().toString());
+        data.insert(QLatin1String(CPP_TOOLCHAINPATH), mainFilePath.absolutePath().path());
 
-        if (auto gcc = dynamic_cast<ProjectExplorer::GccToolChain *>(mainTc)) {
+        if (auto gcc = mainTc->asGccToolchain()) {
             QStringList compilerFlags = gcc->platformCodeGenFlags();
             filterCompilerLinkerFlags(targetAbi, compilerFlags);
             data.insert(QLatin1String(CPP_PLATFORMCOMMONCOMPILERFLAGS), compilerFlags);
@@ -368,13 +360,13 @@ QVariantMap DefaultPropertyProvider::autoGeneratedProperties(const ProjectExplor
         }
         if (targetAbi.os() == ProjectExplorer::Abi::DarwinOS) {
             // Reverse engineer the Xcode developer path from the compiler path
-            const QRegularExpression compilerRe(
+            static const QRegularExpression compilerRe(
                 QStringLiteral("^(?<developerpath>.*)/Toolchains/(?:.+)\\.xctoolchain/usr/bin$"));
-            const QRegularExpressionMatch compilerReMatch = compilerRe.match(cxxCompilerPath.absolutePath().toString());
+            const QRegularExpressionMatch compilerReMatch = compilerRe.match(cxxCompilerPath.absolutePath().toUrlishString());
             if (compilerReMatch.hasMatch()) {
                 const QString developerPath = compilerReMatch.captured(QStringLiteral("developerpath"));
                 data.insert(QLatin1String(XCODE_DEVELOPERPATH), developerPath);
-                toolchain.insert(0, QStringLiteral("xcode"));
+                toolchain = "xcode";
 
                 // If the sysroot is part of this developer path, set the canonical SDK name
                 const QDir sysrootdir(QDir::cleanPath(sysroot));
@@ -403,7 +395,7 @@ QVariantMap DefaultPropertyProvider::autoGeneratedProperties(const ProjectExplor
     }
 
     if (!toolchain.isEmpty())
-        data.insert(QLatin1String(QBS_TOOLCHAIN), toolchain);
+        data.insert("qbs.toolchainType", toolchain);
 
     return data;
 }
